@@ -1,4 +1,6 @@
 
+
+
 RMstrokorbMono <- function(phi) stop("Please use 'RMm2r' instead")
 RMstrokorbBall <- function(phi) stop("Please use 'RMm3b' instead")
 RMstrokorbPoly <- function(phi) stop("Please use 'RMmps' instead")
@@ -180,7 +182,7 @@ GetSymbols <- function(ll) {
        
 
 RMuser <- function(type, domain, isotropy, vdim, beta,
-                   variab.names = c("x", "y", "z", "T"),
+                   varnames = c("x", "y", "z", "T"),
                    fctn, fst, snd, envir, var, scale, Aniso, proj) {
 	cl <- match.call()
 	submodels <- par.general <- par.model <- list() 
@@ -194,9 +196,9 @@ RMuser <- function(type, domain, isotropy, vdim, beta,
         if (any(par.model[['type']] < ProcessType))
           message("It is likely that the function you are defining is already available in 'RandomFields', or hasn't got the claimed property ('",
                   TYPENAMES[1+par.model[['type']]],
-                  ")'. (If you are not sure whether your function is positive/negative definite, please contact schlather@math.uni-mannheim.de.)\nUsing predefined functions leads to (much!) shorter computing times.\nSee ?RMmodels for an overview over the implemented models. Further,\nsome simulation methods do not work at all for user defined functions.")
+                  "'). (If you are not sure whether your function is positive/negative definite, please contact schlather@math.uni-mannheim.de.)\nUsing predefined functions leads to (much!) shorter computing times (up to a factor 100).\nSee ?RMmodels for an overview over the implemented models. Further,\nsome simulation methods do not work at all for user defined functions.")
         else if (any(par.model[['type']] == TrendType))
-          message("Please make sure that the defined function is not available in 'RandomFields'.\nUsing predefined functions leads to (much!) shorter computing times. Further,\nsome simulation methods do not work at all for user defined functions.");
+          message("Please make sure that the defined function is not available in 'RandomFields'.\nUsing predefined functions leads to (much!) shorter computing times (up to a factor 100).\nSee ?RMmodelsTrend  for an overview over the implemented models. Further,\nsome simulation methods do not work at all for user defined functions.");
 
 	if (hasArg(domain)) {
 	  if (is.numeric(domain)) par.model[['domain']] <- domain
@@ -227,14 +229,14 @@ RMuser <- function(type, domain, isotropy, vdim, beta,
 	}
  	if (hasArg(fctn)) {
           f <- substitute(fctn)
-          par <- variab.names %in% GetSymbols(as.list(as.list(f)[-1]))
+          par <- varnames %in% GetSymbols(as.list(as.list(f)[-1]))
           par.model[['fctn']] <- f
  	} else stop("'fctn' not given")
        
 	if (hasArg(fst)) {
           f <- substitute(fst)
           if (any(xor(par,
-                      variab.names %in% GetSymbols(as.list(as.list(f)[-1])))))
+                      varnames %in% GetSymbols(as.list(as.list(f)[-1])))))
             stop("the variables in 'fst' do not match the ones in 'fctn'")
           par.model[['fst']] <- f
 	} else if (hasArg(snd)) stop("'fst' not given")
@@ -242,13 +244,13 @@ RMuser <- function(type, domain, isotropy, vdim, beta,
 	if (hasArg(snd)) {         
 	  f <- substitute(snd)          
           if (any(xor(par,
-                      variab.names %in% GetSymbols(as.list(as.list(f)[-1])))))
+                      varnames %in% GetSymbols(as.list(as.list(f)[-1])))))
             stop("the variables in 'snd' do not match the ones in 'fctn'")
 	  par.model[['snd']] <- f
 	}
 
         ##      Print(par.names, par);        xxx
-        par.model[['variab.names']] <- which(par)    
+        par.model[['varnames']] <- which(par)    
 	par.model[['envir']] <- if (hasArg(envir)) envir else new.env()
 	par.general[['var']] <-if (hasArg(var)) var else ZF_DEFAULT_STRING
 	par.general[['scale']] <-if (hasArg(scale)) scale else ZF_DEFAULT_STRING
@@ -275,4 +277,46 @@ RMuser <- new('RMmodelgenerator',
               )
 
 
-f <- substitute(exp(-x-y+z(), zz=a))
+
+CheckProj <- function(arg, subst) {
+  u <- try(is.numeric(arg) || is.logical(arg) || is.language(arg)
+           || is.character(arg)
+           || is.list(arg) || is(arg, class2='RMmodel'), silent=TRUE)
+  if (is.logical(u) && u) {
+    if (is.character(arg)) {
+      if (length(arg) != 1) stop("'proj' must be a single string")
+      
+      arg <- pmatch(arg, PROJECTION_NAMES) - 1 - length(PROJECTION_NAMES)
+      if (is.na(arg)) stop("'proj': unknown character sequence")
+    }
+    arg
+  }
+  else if (substr(deparse(subst), 1, 1)=='R') arg
+  else  do.call('RRdistr', list(subst))
+}
+
+
+CheckArg <- function(arg, subst, distr) {
+  u <- try(is.numeric(arg) || is.logical(arg) || is.language(arg)
+           || is.list(arg) || is(arg, class2='RMmodel'), silent=TRUE)
+  if (is.logical(u) && u) arg
+  else if (substr(deparse(subst), 1, 1)=='R') arg
+  else if (distr) do.call('RRdistr', list(subst))
+  else stop('random parameter not allowed')
+}
+
+
+CheckChar <- function(arg, subst, names, distr) {
+  if (is.character(arg)) {
+    a <- pmatch(arg, names) - 1
+    if (any(is.na(a))) stop('unknown string value')
+    return(a)
+  }
+  u <- try(is.numeric(arg) || is.logical(arg) || is.language(arg)
+           || is.list(arg) || is(arg, class2='RMmodel'), silent=TRUE)
+  if (is.logical(u) && u) arg
+  else if (substr(deparse(subst), 1, 1)=='R') arg
+  else if (distr) do.call('RRdistr', list(subst))
+  else stop('random parameter not allowed')
+}
+
